@@ -1,5 +1,8 @@
 import json
 from pathlib import Path
+from typing import NamedTuple
+
+Key = str
 
 
 def is_hash(path: Path):
@@ -16,7 +19,25 @@ def from_hash(path: Path):
     return path.with_name(path.stem)
 
 
-def load_tree_hash(path: Path):
+class FileHash(NamedTuple):
+    key: Key
+    path: Path
+    hash: Path
+
+
+class TreeHash(NamedTuple):
+    key: Key
+    root: Path
+    hash: Path
+    relative: Path
+
+
+def load_key(path: Path):
+    with open(path, 'r') as file:
+        return file.read().strip()
+
+
+def load_tree(path: Path):
     with open(path, 'r') as file:
         return json.load(file)
 
@@ -24,7 +45,12 @@ def load_tree_hash(path: Path):
 # FIXME: the names are misleading
 def load_tree_key(path: Path):
     with open(path) as file:
-        key = file.read().strip()
-        if key.startswith('tree:'):
-            key = key[5:]
-        return key
+        return strip_tree(file.read().strip())
+
+
+def strip_tree(key):
+    if key.startswith('tree:'):
+        key = key[5:]
+    elif key.startswith('T:'):
+        key = key[2:]
+    return key
