@@ -1,35 +1,62 @@
+import inspect
 import os
 import subprocess
+import warnings
 from functools import lru_cache
 from pathlib import Path
-from typing import Sequence
+from typing import Sequence, Union
 
 from wcmatch import glob
-from connectome.storage import Storage
 
 from .config import CONFIG, build_storage, find_vcs_root
 from .hash import is_hash, to_hash, load_tree, load_tree_key, strip_tree, FileHash, TreeHash, Key
-from .local import Local
+from .local import Local, LocalVersion
 from .utils import InconsistentRepositories, call_git, HashNotFound, PathLike, RepositoryNotFound
 
-# TODO: add `Local` here
-Version = str
+CommittedVersion = str
+Version = Union[CommittedVersion, LocalVersion]
 
 
 class Repository:
-    def __init__(self, root: PathLike, storage: Storage, cache: PathLike = None):
-        self.storage = storage
-        self.root = Path(root)
-        self.cache = cache
+    """
+    Interface that represents a `bev` repository.
+
+    Parameters
+    ----------
+    root:
+        the path to the repository's root, i.e. the folder that contains `.bev.config`
+    fetch: bool
+        whether to fetch files from remote locations when needed. Can be overridden in corresponding methods
+    version: str, Local
+        default value for data version. Can be either a string with a commit hash/tag or the `Local` object.
+        Can be overridden in corresponding methods
+    """
+
+    def __init__(self, *root: PathLike, fetch: bool = True, version: Version = None):
+        self.root = Path(*root)
+        self.storage, self.cache = build_storage(self.root)
+        self._fetch, self._version = fetch, version
 
     @classmethod
     def from_root(cls, *parts: PathLike):
-        root = Path(*parts)
-        storage, cache = build_storage(root)
-        return cls(root, storage, cache)
+        warnings.warn(
+            'This classmethod is deprecated, call the constructor directly for the same behaviour', UserWarning)
+        return cls(*parts)
 
     @classmethod
-    def from_vcs(cls, *parts: PathLike):
+    def from_here(cls, *relative: PathLike, fetch: bool = True, version: Version = None) -> 'Repository':
+        """
+        Creates a repository with a path `relative` to the file in which this method is called.
+
+        Examples
+        --------
+        >>> repo = Repository.from_here('../../data')
+        """
+        file = Path(inspect.stack()[1].filename)
+        return cls(file.parent / Path(*relative), fetch=fetch, version=version)
+
+    @classmethod
+    def from_vcs(cls, *parts: PathLike) -> 'Repository':
         vcs = find_vcs_root(Path(os.getcwd(), *parts))
         if vcs is None:
             raise RepositoryNotFound(f'{Path(*parts)} is not inside a vcs repository')
@@ -40,13 +67,13 @@ class Repository:
         if len(configs) > 1:
             raise InconsistentRepositories(f'This vcs repository contains multiple bev repositories: {configs}')
 
-        return cls.from_root(configs[0].parent)
+        return cls(configs[0].parent)
 
     @property
-    def current_version(self):
+    def current_version(self) -> CommittedVersion:
         return self.latest_version()
 
-    def latest_version(self, path: PathLike = '.'):
+    def latest_version(self, path: PathLike = '.') -> CommittedVersion:
         path = Path(path)
         if not (self.root / path).exists() and not is_hash(path):
             path = to_hash(path)
@@ -56,17 +83,19 @@ class Repository:
 
         return call_git(f'git log -n 1 --pretty=format:%H -- {path}', self.root)
 
-    def resolve(self, *parts: PathLike, version: Version) -> Path:
-        key = self.get_key(*parts, version=version)
-        return self.storage.get_path(key)
+    def resolve(self, *parts: PathLike, version: Version = None, fetch: bool = None) -> Path:
+        version = self._resolve_version(version)
+        key = self.get_key(*parts, version=version, fetch=fetch)
+        return self.storage.get_path(key, self._resolve_fetch(fetch))
 
-    def glob(self, *parts: PathLike, version: Version) -> Sequence[Path]:
+    def glob(self, *parts: PathLike, version: Version = None, fetch: bool = None) -> Sequence[Path]:
+        version = self._resolve_version(version)
         h = self._split(Path(*parts), version)
         if not isinstance(h, TreeHash):
             raise ValueError('`glob` is only applicable to tree hashes')
 
         pattern = str(h.relative)
-        tree = self.storage.load(load_tree, h.key)
+        tree = self._load(load_tree, h.key, fetch)
 
         files = set(tree)
         for file in tree:
@@ -76,33 +105,38 @@ class Repository:
         return [h.root / file for file in glob.globfilter(files, pattern, flags=glob.GLOBSTAR)]
 
     # TODO: cache this based on path parents
-    def get_key(self, *parts: PathLike, version: Version) -> Key:
+    def get_key(self, *parts: PathLike, version: Version = None, fetch: bool = None) -> Key:
+        version = self._resolve_version(version)
         h = self._split(Path(*parts), version)
         if isinstance(h, FileHash):
             return h.key
 
         assert isinstance(h, TreeHash), h
         relative = str(h.relative)
-        tree = self._get_tree(h.key, version)
+        tree = self._get_tree(h.key, version, fetch)
         if relative not in tree:
             raise HashNotFound(relative)
 
         return tree[relative]
 
-    def load_tree(self, path: PathLike, version: Version):
+    def load_tree(self, path: PathLike, version: Version = None, fetch: bool = None) -> dict:
+        version = self._resolve_version(version)
         key = self._get_hash(Path(path), version)
         if key is None:
             raise HashNotFound(path)
-        return self._get_tree(key, version)
+        return self._get_tree(key, version, fetch)
 
-    def _get_tree(self, key, version):
+    # internal logic
+
+    def _get_tree(self, key, version, fetch):
+        # we need the version here, because we want to cache only a committed tree
         if version == Local:
-            return self.storage.load(load_tree, key)
-        return self._load_cached_tree(key)
+            return self._load(load_tree, key, fetch)
+        return self._load_cached_tree(key, fetch)
 
     @lru_cache(None)
-    def _load_cached_tree(self, key):
-        return self.storage.load(load_tree, key)
+    def _load_cached_tree(self, key, fetch):
+        return self._load(load_tree, key, fetch=fetch)
 
     def _get_hash(self, path: Path, version: Version):
         if version == Local:
@@ -116,6 +150,7 @@ class Repository:
 
     @lru_cache(None)
     def _get_committed_hash(self, relative: Path, version: str):
+        assert isinstance(version, str), type(version)
         relative = str(relative)
         if not relative.startswith('./'):
             relative = f'./{relative}'
@@ -125,6 +160,10 @@ class Repository:
         except subprocess.CalledProcessError:
             pass
 
+    def _load(self, func, key, fetch):
+        fetch = self._resolve_fetch(fetch)
+        return self.storage.load(func, key, fetch=fetch)
+
     def _split(self, path: Path, version: Version):
         # TODO: use bin-search?
         for parent in list(reversed(path.parents))[1:]:
@@ -140,3 +179,15 @@ class Repository:
             raise HashNotFound(path)
 
         return FileHash(key, path, hash_path)
+
+    def _resolve_fetch(self, fetch):
+        if fetch is None:
+            return self._fetch
+        return fetch
+
+    def _resolve_version(self, version):
+        if version is None:
+            version = self._version
+        if version is None:
+            raise ValueError('The argument `version` must be provided.')
+        return version
